import numpy as np
import numpy.ma as ma

class MaskManager():

    def __init__(self, mask_dict={}, fill_value=9999):
        self.fill_value = fill_value
        self.mask_dict = {key: np.array(mask) for key, mask in mask_dict.items()}
        self._validate_mask()

        ## 0: valid, other value: invalid, 
        # example of mask that fot to get only elements in center block: 
        # 
        # [[1,1,1,1],
        #  [1,0,0,1],
        #  [1,0,0,1].
        #  [1,1,1,1]]

    def get_all(self, target):
        return MaskedAll(target, self.mask_dict, self.fill_value)

    def mask(self, target, mask_name):
        masked = ma.array(target, mask=self.mask_dict[mask_name], fill_value=self.fill_value)
        return masked

    def _validate_mask(self):
        shape = self.mask_dict[self.mask_dict.keys()[0]].shape
        for key, mask in self.mask_dict.items():
            assert mask.shape == shape

    def add_mask(self, name, mask):
        self.mask_dict[name] = mask
        self._validate_mask()

class MaskedAll():
    def __init__(self, target, mask_dict, fill_value):
        self.target = target
        self.mask_dict = mask_dict
        self.fill_value = fill_value
        self.masked_arrays = {key: ma.array(target, mask=mask, fill_value=fill_value) for key,mask in mask_dict.items()}

    def __getitem__(self, name):
        return self.masked_arrays[name].filled()


from collections import OrderedDict

class SimpleSlicer():
    def __init__(self, len_dict):
        '''
        Requirement `len_dict` as orderd dict
        '''
        self.len_dict = OrderedDict(len_dict)
        self.slice_dict = self.create_slice_dict(self.len_dict)

    def apply_element_state(self, key, length):
        self.len_dict[key] = length
        self.slice_dict = self.create_slice_dict(self.len_dict)

    def get_len(self, key):
        assert key in self.len_dict.keys(), 'Not registered "{}". Valid keys for len_dict are {}.'.format(key, self.len_dict.keys())
        return self.len_dict[key]

    def get_slice(self, key):
        assert key in self.slice_dict.keys(), 'Not registered "{}". Valid keys for slice_dict are {}.'.format(key, self.slice_dict.keys())
        return self.slice_dict[key]

    def get_total_len(self):
        '''
        Returns the total length of state excluding variable length.
        '''
        return sum([x for x in self.len_dict.values() if x > 0])

    @staticmethod
    def create_slice_dict(len_dict):
        assert len([x for x in len_dict.values() if x <= 0]) <= 1
        
        start = 0
        slice_dict = OrderedDict()
        for key in len_dict:
            end = start+len_dict[key] if len_dict[key] > 0 else None
            slice_dict[key] = slice(start, end)
            start += len_dict[key]
            
        return slice_dict

    def get(self, rs_state, key):
        assert key in self.slice_dict.keys()
        return rs_state[self.slice_dict[key]]


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['labeled_mask']

package_data = \
{'': ['*']}

extras_require = \
{':python_version >= "2.7.0" and python_version < "2.8.0"': ['numpy>=1.16,<2.0']}

setup_kwargs = {
    'name': 'labeled-mask',
    'version': '0.1.0',
    'description': 'Provide interface of tagged mask that make easy to squeeze target data from array',
    'long_description': None,
    'author': 'wwwshwww',
    'author_email': 'www.shinderu.www@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=2.7',
}


setup(**setup_kwargs)
